package mapReduce;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

public class ArticleParserMapper extends MapReduceBase implements Mapper<LongWritable, Text, Text, Text> {

	private String articleRegex = "\\{(.*?)[^\\[\\]]\\}";
	private String firstArticleField = "title";
	private String secondArticleField = "body";
	
	public static String titleBodySeparator = "&&&&&";
	
    private Text articleTitle = new Text();
    private Text articleBody = new Text();  
	
	private List<String> getSingleArticle(String inputString) {
		
		List<String> listOfSingleArticles = new ArrayList<String>();
		
		Pattern pattern = Pattern.compile(articleRegex); 
		Matcher m = pattern.matcher(inputString);
		while(m.find()) {
            String s = m.group(1);     
            listOfSingleArticles.add(s);  
		}
		
		return listOfSingleArticles;
	}

	private String getArticleFieldContent(String articleString, String articleField) {
		
		String articleFieldContent = "";
		
		int beginIndex = articleString.indexOf(articleField + "\" :");
		int inputStringLength = articleString.length();
		articleFieldContent = articleString.substring(beginIndex, inputStringLength);
		
		int titleNameBeginIndex = articleFieldContent.indexOf(": ");
		
		int endIndex = 0;
		int firstEndIndex = articleFieldContent.indexOf("\",");
		int secondEndIndex = articleFieldContent.indexOf("\" ,");
		if(firstEndIndex > 0 && secondEndIndex > 0) {
			if (firstEndIndex < secondEndIndex) {
				endIndex = firstEndIndex;
			}
			else {
				endIndex = secondEndIndex;
			}
		}
		else {
			if(firstEndIndex < 0 && secondEndIndex > 0) {
				endIndex = secondEndIndex; 
			}
			if(secondEndIndex < 0 && firstEndIndex > 0) {
				endIndex = firstEndIndex;
			}
		}
		
		articleFieldContent = articleFieldContent.substring(titleNameBeginIndex + 1, endIndex + 1);
		articleFieldContent = articleFieldContent.trim();
		
		return articleFieldContent;
	}
	
    @Override
    public void map(LongWritable key, Text value, OutputCollector<Text, Text> output, Reporter reporter) 
    throws IOException{

    	String line = value.toString();
		List<String> articles = getSingleArticle(line);
		
		for (int i = 0; i < articles.size(); i++) {
			
			String tempArticleTitle = getArticleFieldContent(articles.get(i), firstArticleField);
			String tempArticleBody = getArticleFieldContent(articles.get(i), secondArticleField);
			
			StringBuilder articleBodyStringBuilder = new StringBuilder(496);
			articleBodyStringBuilder.append(titleBodySeparator);
			articleBodyStringBuilder.append(tempArticleBody);
			
			articleTitle.set(tempArticleTitle);
			articleBody.set(articleBodyStringBuilder.toString());
			
			output.collect(articleTitle, articleBody);
		}
    }
 }

