package mapReduce;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

public class KeywordSearchMapper extends MapReduceBase implements
		Mapper<LongWritable, Text, Text, Text> {

	private Text outputKeyword = new Text();
	private Text outputKey = new Text();

	public List<String> getArticleKeywords() {

		List<String> articleKeywords = new ArrayList<String>();

		articleKeywords.add("two");
		articleKeywords.add("today");
//		articleKeywords.add("the");
//		articleKeywords.add("for");
//		articleKeywords.add("from");
//		articleKeywords.add("that");

		return articleKeywords;
	}

	public String readKeywordsFromFile() {

		Configuration conf = new Configuration();
		StringBuilder strBuilder = new StringBuilder(496);
		
		try {
			FileSystem fs = FileSystem.get(conf);
			Path inFile = new Path("Keywords");
			
			if (fs.exists(inFile) && fs.isFile(inFile)) {
				BufferedReader br = new BufferedReader(new InputStreamReader(fs.open(inFile)));
				String line = br.readLine();
				while (line != null) {
					strBuilder.append(line);
					line = br.readLine();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return strBuilder.toString();
	}
	
	public List<String> parseInputKeywords() {
		
		String keywords = readKeywordsFromFile();
		String[] keywordsArray = keywords.split("\\s");
		
		List<String> keywordsList = new ArrayList<String>();
		
		for (int i = 0; i < keywordsArray.length; i++) {
			keywordsList.add(keywordsArray[i]);
		}
		
		return keywordsList;
	}

	public String[] getArticleBodyWords(String articleBody) {

		String tempArticleBody = articleBody.replaceAll("[\"\\.\\,\\&\\;]", "");
		String[] articleBodyWords = tempArticleBody.split("\\s");

		return articleBodyWords;
	}

	@Override
	public void map(LongWritable key, Text value,
			OutputCollector<Text, Text> output, Reporter reporter)
			throws IOException {

		String line = value.toString();

		int separateIndex = line
				.indexOf(ArticleParserMapper.titleBodySeparator);
		String articleTitle = line.substring(0, separateIndex);
		String articleBody = line.substring(separateIndex + 5, line.length());

		String[] articleBodyWords = getArticleBodyWords(articleBody);

		//List<String> keywords = parseInputKeywords();
		List<String> keywords = getArticleKeywords();

		outputKey.set(articleTitle);

		for (int i = 0; i < articleBodyWords.length; i++) {
			if (keywords.contains(articleBodyWords[i])) {
				outputKeyword.set(articleBodyWords[i]);
				output.collect(outputKey, outputKeyword);
			}
		}
	}
}
