package mapReduce;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;

public class NewYorkTimesNewsDriver {

	public static int numberOfReducers = 10;
	public static String inputDirectory = "NewYorkTimesArticles";
	public static String tempDirectory = "ArticlesParsedData";
	public static String outputDirectory = "ArticlesOutput";
	public static String sortOutputDirectory = "FilteredArticlesTitles";

	public static void deleteExistingDirectory(String directoryName) {
		try {
			Configuration config = new Configuration();
			FileSystem hdfs = FileSystem.get(config);
			Path path = new Path(directoryName);
			hdfs.delete(path, true);
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public void sendRequestForData(String downloadDirecotry, String searchKeyword, int offset) {
		
		HTTPRequestPoster poster = new HTTPRequestPoster();
		String	newYorkTimesRequest = poster.getNewYorkTimesRequest(searchKeyword, offset);
		String	result = poster.sendGetRequest(newYorkTimesRequest); 
			
			try{
				Path pt = new Path(downloadDirecotry + "/" +"NewYorkTimesArticles" + offset);
				Configuration config = new Configuration();
				FileSystem hdfs = FileSystem.get(config);
				BufferedWriter br = new BufferedWriter(new OutputStreamWriter(hdfs.create(pt,true)));
				br.write(result);
				br.close();
			} catch (Exception e){
				System.out.println("File not found");
	        }
	}
	
	public static void main(String[] args) {

		JobClient client = new JobClient();

		String parserOutputDirectory = tempDirectory;
		deleteExistingDirectory(parserOutputDirectory);

		// Configure the parser map/reduce process
		JobConf conf = new JobConf(mapReduce.NewYorkTimesNewsDriver.class);

		conf.setMapperClass(mapReduce.ArticleParserMapper.class);
		conf.setReducerClass(mapReduce.ArticleParserReducer.class);

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(Text.class);

		conf.setNumReduceTasks(numberOfReducers);

		FileInputFormat.setInputPaths(conf, new Path(inputDirectory));
		FileOutputFormat.setOutputPath(conf, new Path(parserOutputDirectory));

		client.setConf(conf);
		try {
			JobClient.runJob(conf);
		} catch (Exception e) {
			e.printStackTrace();
		}

		String keywordSearchInputDirectory = tempDirectory;
		String keywordSearchOutputDirectory = outputDirectory;
		deleteExistingDirectory(keywordSearchOutputDirectory);
		
		JobConf keywordSearchConf = new JobConf(
				mapReduce.NewYorkTimesNewsDriver.class);

		keywordSearchConf.setMapperClass(KeywordSearchMapper.class);
		keywordSearchConf.setReducerClass(KeywordSearchReducer.class);
		
		keywordSearchConf.setNumReduceTasks(numberOfReducers);

		keywordSearchConf.setOutputKeyClass(Text.class);
		keywordSearchConf.setOutputValueClass(Text.class);

		FileInputFormat.setInputPaths(keywordSearchConf, new Path(keywordSearchInputDirectory));
		FileOutputFormat.setOutputPath(keywordSearchConf, new Path(keywordSearchOutputDirectory));

		client.setConf(keywordSearchConf);
		try {
			JobClient.runJob(keywordSearchConf);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		String sortTitlesInputDirectory = outputDirectory;
		String sortTitlesOutputDirectory = sortOutputDirectory;
		deleteExistingDirectory(sortTitlesOutputDirectory);
		
		JobConf sortConf = new JobConf(mapReduce.SortArticleTitlesDriver.class);

		sortConf.setMapperClass(SortArticleTitlesMapper.class);
		sortConf.setReducerClass(SortArticleTitlesReducer.class);
		
		sortConf.setOutputKeyClass(Text.class);
		sortConf.setOutputValueClass(Text.class);

		FileInputFormat.setInputPaths(sortConf, new Path(sortTitlesInputDirectory));
		FileOutputFormat.setOutputPath(sortConf, new Path(sortTitlesOutputDirectory));

		client.setConf(sortConf);
		try {
			JobClient.runJob(sortConf);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
